@@ -16,20 +16,27 @@
 #ifndef NETWORKJOBS_H
 #define NETWORKJOBS_H
 
+#include <QBuffer>
+
 #include "abstractnetworkjob.h"
 
 #include "common/result.h"
 
-#include <QBuffer>
-#include <QUrlQuery>
-#include <QJsonDocument>
-#include <functional>
-
 class QUrl;
+class QUrlQuery;
 class QJsonObject;
+class QJsonDocument;
+class QDomDocument;
 
 namespace OCC {
 
+constexpr auto HttpErrorCodeNone = 0;
+constexpr auto HttpErrorCodeSuccess = 200;
+constexpr auto HttpErrorCodeSuccessCreated = 201;
+constexpr auto HttpErrorCodeSuccessNoContent = 204;
+constexpr auto HttpErrorCodeBadRequest = 400;
+constexpr auto HttpErrorCodeUnsupportedMediaType = 415;
+
 struct HttpError
 {
     int code; // HTTP error code
@@ -133,8 +140,8 @@ class OWNCLOUDSYNC_EXPORT LsColJob : public AbstractNetworkJob
 {
     Q_OBJECT
 public:
-    explicit LsColJob(AccountPtr account, const QString &path, QObject *parent = nullptr);
-    explicit LsColJob(AccountPtr account, const QUrl &url, QObject *parent = nullptr);
+    explicit LsColJob(AccountPtr account, const QString &path);
+    explicit LsColJob(AccountPtr account, const QUrl &url);
     void start() override;
     QHash<QString, ExtraFolderInfo> _folderInfos;
 
@@ -147,7 +154,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QList<QByteArray> properties);
-    QList<QByteArray> properties() const;
+    [[nodiscard]] QList<QByteArray> properties() const;
 
 signals:
     void directoryListingSubfolders(const QStringList &items);
@@ -189,7 +196,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QList<QByteArray> properties);
-    QList<QByteArray> properties() const;
+    [[nodiscard]] QList<QByteArray> properties() const;
 
 signals:
     void result(const QVariantMap &values);
@@ -199,6 +206,10 @@ private slots:
     bool finished() override;
 
 private:
+    static QVariantMap processPropfindDomDocument(const QDomDocument &domDocument);
+    static QStringList processTagsInPropfindDomDocument(const QDomDocument &domDocument);
+    static QVariantList processSystemTagsInPropfindDomDocument(const QDomDocument &domDocument);
+
     QList<QByteArray> _properties;
 };
 
@@ -265,7 +276,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QMap<QByteArray, QByteArray> properties);
-    QMap<QByteArray, QByteArray> properties() const;
+    [[nodiscard]] QMap<QByteArray, QByteArray> properties() const;
 
 signals:
     void success();
@@ -347,7 +358,7 @@ private slots:
     void slotRedirected(QNetworkReply *reply, const QUrl &targetUrl, int redirectCount);
 
 private:
-    bool _subdirFallback;
+    bool _subdirFallback = false;
 
     /** The permanent-redirect adjusted account url.
      *
@@ -357,7 +368,35 @@ private:
     QUrl _serverUrl;
 
     /** Keep track of how many permanent redirect were applied. */
-    int _permanentRedirects;
+    int _permanentRedirects = 0;
+};
+
+/**
+ * @brief The CheckRedirectCostFreeUrlJob class
+ * @ingroup libsync
+ */
+class OWNCLOUDSYNC_EXPORT CheckRedirectCostFreeUrlJob : public AbstractNetworkJob
+{
+    Q_OBJECT
+public:
+    explicit CheckRedirectCostFreeUrlJob(const AccountPtr &account, QObject *parent = nullptr);
+    void start() override;
+
+signals:
+    /**
+    * a check is finished
+    * \a statusCode cost-free URL GET HTTP response code
+    */
+    void jobFinished(int statusCode);
+    /** A timeout occurred.
+     *
+     * \a url The specific url where the timeout happened.
+     */
+    void timeout(const QUrl &url);
+
+private:
+    bool finished() override;
+    void onTimedOut() override;
 };
 
 
@@ -419,10 +458,10 @@ Q_SIGNALS:
 protected:
     bool finished() override;
 
-    QNetworkRequest& request();
-    QByteArray& body();
-    QUrlQuery& additionalParams();
-    QByteArray verbToString() const;
+    [[nodiscard]] QNetworkRequest& request();
+    [[nodiscard]] QByteArray& body();
+    [[nodiscard]] QUrlQuery& additionalParams();
+    [[nodiscard]] QByteArray verbToString() const;
 
 private:
     QByteArray _body;
@@ -491,7 +530,6 @@ public:
         WebViewFlow,
 #endif // WITH_WEBENGINE
         Basic, // also the catch-all fallback for backwards compatibility reasons
-        OAuth,
         LoginFlowV2
     };
     Q_ENUM(AuthType)
@@ -499,7 +537,7 @@ public:
     explicit DetermineAuthTypeJob(AccountPtr account, QObject *parent = nullptr);
     void start();
 signals:
-    void authType(AuthType);
+    void authType(OCC::DetermineAuthTypeJob::AuthType);
 
 private:
     void checkAllDone();
@@ -511,10 +549,11 @@ private:
     bool _getDone = false;
     bool _propfindDone = false;
     bool _oldFlowDone = false;
+    bool useFlow2 = false;
 };
 
 /**
- * @brief A basic job around a network request without extra funtionality
+ * @brief A basic job around a network request without extra functionality
  * @ingroup libsync
  *
  * Primarily adds timeout and redirection handling.
